package com.flyfish.framework.service;

import com.flyfish.framework.domain.po.User;
import com.flyfish.framework.repository.ReactiveUserRepository;
import com.flyfish.framework.service.impl.BaseReactiveServiceImpl;
import com.flyfish.framework.utils.RedisOperations;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.data.redis.core.ReactiveRedisTemplate;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;

/**
 * 异步用户service
 *
 * @author wangyu
 */
@Service
public class ReactiveUserService extends BaseReactiveServiceImpl<User> {

    @Resource
    private RedisOperations redisOperations;

    /**
     * 获取用户数据
     *
     * @param username 用户
     * @return 结果
     */
    @Cacheable
    public Mono<User> findByUsername(String username) {
        if (redisOperations.hasKey(getCacheKey(username))) {
            return
        }
        return ((ReactiveUserRepository) repository).findByUsername(username);
    }

    private String getCacheKey(String username) {
        return "user-" + username;
    }

}
