package com.flyfish.framework.service;

import com.flyfish.framework.auditor.BeanAuditor;
import com.flyfish.framework.domain.po.User;
import com.flyfish.framework.enums.UserType;
import com.flyfish.framework.repository.UserRepository;
import com.flyfish.framework.service.impl.BaseServiceImpl;
import com.flyfish.framework.utils.Assert;
import com.flyfish.framework.utils.StrengthUtils;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang3.StringUtils;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Optional;

@Service
public class UserService extends BaseServiceImpl<User> implements UserFindService {

    @Resource
    private PasswordEncoder passwordEncoder;

    /**
     * 获取用户数据
     *
     * @param username 用户
     * @return 结果
     */
    @Override
    public Optional<User> findByUsername(String username) {
        return ((UserRepository) repository).findByUsername(username);
    }

    @Component
    @RequiredArgsConstructor
    public static class Auditor implements BeanAuditor<User> {

        private final PasswordEncoder passwordEncoder;

        /**
         * 对实体进行审查，并补全相关字段
         *
         * @param entity 原数据
         */
        @Override
        public void audit(User entity) {
            if (null == entity.getId() && StringUtils.isNotBlank(entity.getPassword())) {
                Assert.isTrue(StrengthUtils.isValid(entity.getPassword()), "密码强度不够，至少应该包含数字、大小写字母、符号组合");
                entity.setPassword(passwordEncoder.encode(entity.getPassword()));
            }
            if (null == entity.getType()) {
                entity.setType(UserType.ADMIN);
            }
            if (null == entity.getEnable()) {
                entity.setEnable(true);
            }
            if (null == entity.getApp()) {
                entity.setApp(false);
            }
            if (null == entity.getCode()) {
                entity.setCode(entity.getUsername());
            }
        }
    }
}
